(*------------------------------------------------------------------------*)
(*                                                                        *)
(*  CWRUN.mod - Modula-2 Interface to the Clarion for Windows Runtime Lib *)
(*                                                                        *)
(*  This module corrects argument usage and utilizes string stack calls.  *)
(*                                                                        *)
(*  Updated for Clarion V4.0b from Builtins.Clw  C.W.Sumner  12-10-98     *)
(*                                                                        *)
(*  Copyright (c) 1999-2002 Carl W. Sumner                                *)
(*                                                                        *)
(*  WARNING: Any mismatch in these definitions will cause a GPF crash.    *)
(*           This is a pre-release version, use at your own risk.         *)
(*                                                                        *)
(*  Modified for Clarion V5.0a  CWS 06/03/99                              *)
(*  Modified for Clarion V5.5g  CWS 02/20/02                              *)
(*                                                                        *)
(*------------------------------------------------------------------------*)

(*#call (o_a_copy=>off) *)
IMPLEMENTATION MODULE CWRUN;

(* SIGNED,UNSIGNED and Modula-2 WORD are set by __32BIT__ to 2 or 4 bytes *)

IMPORT Lib, Clw;




PROCEDURE PushString(s: ARRAY OF BYTE);
VAR Lth: CARDINAL;
BEGIN
  Lth := Lib.ScanR(ADR(s),HIGH(s)+1,0);
  Clw.Cla$PushString(s, VAL(UNSIGNED, Lth));
END PushString;



PROCEDURE PushBytAry(s: ARRAY OF BYTE);                     (* 02-23-00 *)
VAR Lth: CARDINAL;
BEGIN
  Lth := HIGH(s)+1;
  Clw.Cla$PushString(s, VAL(UNSIGNED, Lth));
END PushBytAry;



PROCEDURE PushOptString(s: ARRAY OF BYTE): BYTE;             (* 04-08-99 *)
VAR Lth: CARDINAL;
BEGIN
  IF (ADR(s) = NIL) OR (s[0] = BYTE(0)) THEN
    Clw.Cla$PushString(NIL^, 0);
    RETURN 1;   (* false *)
  ELSE
    PushString(s);
    RETURN 0;   (* true *)
  END;
END PushOptString;



PROCEDURE PopPString(VAR s: ARRAY OF BYTE);
BEGIN
  Clw.Cla$PopPString(s, VAL(UNSIGNED, SIZE(s)));
END PopPString;



PROCEDURE PopCString(VAR s: ARRAY OF BYTE);
BEGIN
  Clw.Cla$PopCString(s, VAL(UNSIGNED, SIZE(s)));
END PopCString;



PROCEDURE PopString(VAR s: ARRAY OF BYTE);
BEGIN
  Clw.Cla$PopString(s, VAL(UNSIGNED, SIZE(s)));
END PopString;



PROCEDURE FILEERROR(): ErrStr;                               (* 05-25-99 *)
VAR EStr: ErrStr;
BEGIN
  Clw.Cla$FILEERRORMSG();
  PopString(EStr);
  RETURN EStr;
END FILEERROR;



PROCEDURE FILEERRORCODE(): ErrStr;                           (* 05-25-99 *)
VAR EStr: ErrStr;
BEGIN
  Clw.Cla$FILEERRORCODE();
  PopString(EStr);
  RETURN EStr;
END FILEERRORCODE;



PROCEDURE BLANK(X, Y, Width, Height:SIGNED);                 (* 04-15-99 *)
BEGIN
  Clw.Cla$BLANK(X, 0, Y, 0, Width, 0, Height, 0);
END BLANK;



PROCEDURE MESSAGE(s: ARRAY OF CHAR);                         (* 04-16-99 *)
VAR X,Y,Z: BYTE;  Ret: UNSIGNED;
BEGIN
  Z := PushOptString("");   (* button *)
  Y := PushOptString("");   (* icon *)
  X := PushOptString("");   (* caption *)
  PushString(s);            (* string *)
  Ret := Clw.Cla$MessageBox(X,Y,Z,0,0);  (* fl,fl,fl,default,style *)
END MESSAGE;



PROCEDURE MESSAGEBOX(b,i,c,s: ARRAY OF CHAR);                (* 05-24-99 *)
VAR X,Y,Z: BYTE;  Ret: UNSIGNED;
BEGIN
  Z := PushOptString(b);   (* button *)
  Y := PushOptString(i);   (* icon *)
  X := PushOptString(c);   (* caption *)
  PushString(s);           (* string *)
  Ret := Clw.Cla$MessageBox(X,Y,Z,0,0);  (* fl,fl,fl,default,style *)
END MESSAGEBOX;



PROCEDURE Show(X:SIGNED; Y:SIGNED; S: ARRAY OF CHAR);        (* 04-14-99 *)
BEGIN
  PushString(S);
  Clw.Cla$SHOW(X, Y);
END Show;



PROCEDURE TYPESTR(S: ARRAY OF CHAR);                         (* 04-15-99 *)
BEGIN
  PushString(S);
  Clw.Cla$StackTYPE;
END TYPESTR;



PROCEDURE ERRORCODE(): CARDINAL;                      (* 06-04-99 *)
BEGIN
(*%T __32BIT__*)   (* or __WIN32 *)
  RETURN VAL(CARDINAL, Clw.Cla$ERRCODE^);             (* 02-20-02 *)
(*%E*)
(*%F __32BIT__*)
  RETURN VAL(CARDINAL, Clw.Cla$ERRCODE);              (* 06-04-99 *)
(*%E*)
END ERRORCODE;



PROCEDURE ERROR(): ErrStr;
VAR EStr: ErrStr;
BEGIN
  Clw.Cla$StackErrstr();
  PopString(EStr);
  RETURN EStr;
END ERROR;



PROCEDURE ERRORFILE(): ErrStr;
VAR EStr: ErrStr;
BEGIN
  Clw.Cla$ERRORFILE();
  PopString(EStr);
  RETURN EStr;
END ERRORFILE;



PROCEDURE HALT(ret: UNSIGNED; Msg:ARRAY OF CHAR);    (* 06-02-99 *)
VAR X: BYTE;
BEGIN
  X := PushOptString(Msg);                    (* 04-23-99 *)
  Clw.Cla$HALT(VAL(UNSIGNED,ret),X);          (* 04-23-99 *)
END HALT;



PROCEDURE RUN(App:ARRAY OF CHAR; Wait:SIGNED);
VAR rt: LONGINT;
BEGIN
  PushString(App);
  Clw.Cla$RUN(Wait,0);
  rt := Clw.Cla$RUNCODE();
END RUN;



PROCEDURE DDEclient(application,topic:ARRAY OF CHAR): LONGCARD;
VAR X,Y: BYTE;
BEGIN
  Y := PushOptString(topic);
  X := PushOptString(application);
  RETURN Clw.Cla$DDEclient(X,Y);
END DDEclient;



PROCEDURE DDEserver(application,topic:ARRAY OF CHAR): LONGCARD;
VAR X,Y: BYTE;
BEGIN
  Y := PushOptString(topic);
  X := PushOptString(application);
  RETURN Clw.Cla$DDEserver(X,Y);
END DDEserver;



PROCEDURE DDEread(channel:LONGCARD; mode:UNSIGNED; Item:ARRAY OF CHAR; VAR variable:ARRAY OF BYTE);
BEGIN
  PushString(Item);
  Clw.Cla$DDEread(channel, mode, variable);
END DDEread;



PROCEDURE DDEwrite(channel:LONGCARD; mode:UNSIGNED; Item:ARRAY OF CHAR; VAR variable:ARRAY OF BYTE);
BEGIN
  PushString(Item);
  Clw.Cla$DDEwrite(channel, mode, variable);
END DDEwrite;



PROCEDURE DDEpoke(channel:LONGCARD; Item:ARRAY OF CHAR; VAR value:ARRAY OF BYTE);
BEGIN
  PushString(Item);
  Clw.Cla$DDEpoke(channel);
  PopString(value);
END DDEpoke;



PROCEDURE DDEexecute(channel:LONGCARD; command:ARRAY OF BYTE);
BEGIN
  PushString(command);
  Clw.Cla$DDEexecute(channel);
END DDEexecute;



PROCEDURE DDEquery(application,topic:ARRAY OF CHAR);
VAR X,Y: BYTE;
BEGIN
  Y := PushOptString(topic);
  X := PushOptString(application);
  Clw.Cla$DDEquery(X,Y);
END DDEquery;



PROCEDURE DDEitem(VAR item:ARRAY OF CHAR);
BEGIN
  Clw.Cla$DDEitem();
  PopString(item);
END DDEitem;



PROCEDURE DDEapp(VAR App:ARRAY OF CHAR);
BEGIN
  Clw.Cla$DDEapp();
  PopString(App);
END DDEapp;



PROCEDURE DDEtopic(VAR topic:ARRAY OF CHAR);
BEGIN
  Clw.Cla$DDEtopic();
  PopString(topic);
END DDEtopic;



PROCEDURE DDEvalue(VAR value:ARRAY OF CHAR);
BEGIN
  Clw.Cla$DDEvalue();
  PopString(value);
END DDEvalue;





(* No init block *)
END CWRUN.
